/*
 *   Copyright (C) 2008 Matthias Fuchs <mat69@gmx.net>
 *   Copyright (C) 2018 Yuriy Zhyromskiy <gedgeck@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 3 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

// You can debug the code with "plasmoidviewer -a org.kde.plasma.comic"
// Package: zip -r 8-bittheater.comic contents/code/main.es metadata.desktop icon.png
// Install: kpackagetool5 -i 8-bittheater.comic
// Uninstall: kpackagetool5 -r 8-bittheater

// COMMON
baseUrl = "http://www.nuklearpower.com";
defaultUrl = baseUrl + "/8-bit-theater";

// PARSE
baseParseUrl = "http://www\.nuklearpower\.com/";
parseEmptySpaces = "[\n\t\v ]+";
parsePreviousEnding = "\"(.*)=\"prev\"";
parseNextEnding = "\"(.*)=\"next\"";
anyButQuote = "[^\"]*";

function init()
{
    comic.comicAuthor = "Brian Clevinger";
    comic.firstIdentifier = "2001/03/02/episode-001-were-going-where/";
    comic.lastIdentifier = "2010/06/01/the-epilogue/";
    comic.shopUrl = "http://www.splitreason.com/nuklearpower";
    comic.websiteUrl = "http://www.nuklearpower.com/";

    //check if the comic.identifier is empty, there are other ways as well like checking its length
    if (comic.identifier != new String()) {
        comic.websiteUrl += comic.identifier;
        comic.requestPage(comic.websiteUrl, comic.Page);
    } else {
        comic.requestPage(comic.websiteUrl + comic.firstIdentifier, comic.User);
    }
}

function previousId(data){
    var expStr =
        "<a" +
        parseEmptySpaces +
        "href=\"" + 
        baseParseUrl + 
        "(" + anyButQuote + ")" +
        parsePreviousEnding;
    re = new RegExp(
        expStr
    );
    match = re.exec(data);
    if (match != null) {
        var val = match[1];
        if (val !== undefined) {
            comic.previousIdentifier = val;
        }
    }
}

function nextId(data){
    var expStr = 
        "<a" +
        parseEmptySpaces + 
        "href=\"" +
        baseParseUrl +
        "(" + anyButQuote + ")" +
        parseNextEnding;
    re = new RegExp(
        expStr
    );
    match = re.exec(data);
    if (match !== null) {
        comic.nextIdentifier = match[1];
    }
}

function pageRetrieved(id, data)
{
    //find the most recent comic
    if (id === comic.User) {
        var re = new RegExp("<a href=\"http://www\.nuklearpower\.com/([^\"]+)\" rel=\"first\">First</a>");
        var match = re.exec(data);
        if (match !== null) {
            comic.firstIdentifier = match[1];
            comic.identifier = comic.firstIdentifier;
            comic.websiteUrl += comic.identifier;
            comic.requestPage(comic.websiteUrl, comic.Page);
        } else {
            comic.error();
        }
    }
    //get the comic image and all other available information
    if (id === comic.Page) {
        //getting the image and [probably] the title
        re = new RegExp(
            "<div id=\"comic\">[^\"]+<img[^\"]+src=\"([^\"]+)\".*title=\"(.*)\".*>[^\"]+<\/div>"
        );
        match = re.exec(data);
        if (match !== null) {
            //if there still no strip title get it from 'alt' or 'title' option
            if (comic.title.length < 1) {
                if (match[2].length) {
                    comic.title = match[2];
                }
            }
            comic.websiteUrl=defaultUrl;
            comic.requestPage(match[1], comic.Image);
            //update navigation
            previousId(data);
            nextId(data);
        } else {
            comic.error();
        }
    }
}
