/* Tom des Tages comic strip for the KDE comic widget
 * (C) 2009 - Christian Schneider <chresan@fastmail.fm>
 */

/* Writing a comic plugin:
 * 	http://techbase.kde.org/Development/Tutorials/Plasma/ComicPlugin
 */

// A date object containing the date of the first tom-comic.
// (URL to the first tom-comic: http://www.taz.de/digitaz/.tom/tomdestages?day=1995/04/01 )
const TOM_FIRST_COMIC_DATE = date.fromString("1995/04/01","yyyy/MM/dd");
const TOM_HOST_URL = "http://www.taz.de/"
const TOM_SHOP_URL = "http://shop.taz.de/";
const TOM_DES_TAGES_URL = TOM_HOST_URL+"digitaz/.tom/tomdestages";
const TOM_LINK_TO_HTML_BY_DAY_PREFIX = TOM_HOST_URL+"digitaz/.tom/tomdestages?day=";
const TOM_DEBUG_ENABLED = false;
const TOM_SELECT_PREVIOUS = -1;
const TOM_SELECT_NEXT = +1;

/**
 * Prints a debug message if and only if TOM_DEBUG_ENABLED is set.
 */
function tom_debug_message(message) {
  if (TOM_DEBUG_ENABLED) {
    print("DEBUG: "+message);
  }
}

/**
 * Returns the date of the first tom-comic.
 */
function tom_get_first_comic_date() {
  return TOM_FIRST_COMIC_DATE;
}

/**
 * Returns the date of the latest tom-comic.
 */
function tom_get_last_comic_date() {
  return date.currentDate();
}

/**
 * Returns client info metadata. When downloading the comic image 
 * the HTTP Referer must point to the website URL (TOM_LINK_TO_HTML_BY_DAY_PREFIX + date).
 */
function tom_get_client_info(referrerURL) {
  return {
        "User-Agent": "Mozilla/5.0 (compatible; Konqueror/3.5; Linux) KHTML/3.5.6 (like Gecko)",
        "Accept": "text/html, image/jpeg, image/png, image/gif, text/*, image/*, */*",
        "Accept-Encoding": "functionlate",
        "Accept-Charset": "iso-8859-1, iso-8859-15, utf-8;q=0.5, *;q=0.5",
        "Accept-Language": "en",
        "Host": "taz.de",
        "Connection": "Keep-Alive",
	"referrer":referrerURL
  };
}

/**
 * Returns the image URL for a specified date.
 * This function is currently not used.
 * @note The URL is not valid if the date is a Sunday or National Holiday in Germany.
 */
/*
function tom_imageurl_for_date(date_value) {
  var year = date_value.year();
  var month = date_value.month();
  var day = date_value.day();
  var url_date_format = ""+((new Date(year, month-1, day, 0, 0, 0, 0)).getTime()/1000);
  return "http://www.taz.de/dx/.1/gif.t,tom.d,"+url_date_format;
}*/

/**
 * Returns the URL to the html page for the specified date.
 */
function tom_date_to_website_url(date_value) {
  return TOM_LINK_TO_HTML_BY_DAY_PREFIX + date_value.toString("yyyy/MM/dd");
}

/**
 * Returns true if and only if a comic is avaialable on the specified day.
 */
function tom_is_date_in_range(date_value) {
  return (tom_get_first_comic_date().daysTo(date_value)>=0) 
      && (date_value.daysTo(tom_get_last_comic_date())>=0);
}

/**
 * Increments the specified date by the specified number of days.
 * If the date value is out of range then false is returned.
 * Use TOM_SELECT_NEXT/TOM_SELECT_PREVIOUS as offset to get the
 * next/previous date.
 * @date_value the date 
 * @offset_in_days the number of days (can be positive or negative or zero)
 */
function incrementDate(date_value, offset_in_days) {
  var new_date_value = date_value.addDays(offset_in_days);
  return tom_is_date_in_range(new_date_value) ? new_date_value : false;
}

/**
 * Convenience function which returns a string representation of the id argument
 * which is passed to function pageRetrieved(id, data).
 */
function tom_page_retrieved_id_to_string(id) {
  switch (id) {
    case comic.User: return "comic.User";
    case comic.Page: return "comic.Page";
    case comic.Image: return "comic.Image";
    default: return (""+id);
  }
}

/**
 * init() is being called by the dataengine. The dataengine will only call
 * init() if the asked for comic strip has not been cached.
 */
function init() {
  tom_debug_message("function init(): tom comic");
  
  // initialize
  // the first comic
  comic.firstIdentifier = tom_get_first_comic_date();// "1995-04-01";
  // comic plugin author
  comic.comicAuthor = "Christian Schneider"; 
  // comic shop URL
  comic.shopUrl = TOM_SHOP_URL;
  // define next and previous identifier
  comic.nextIdentifier = incrementDate(comic.identifier, TOM_SELECT_NEXT);
  comic.previousIdentifier = incrementDate(comic.identifier, TOM_SELECT_PREVIOUS);
  // the website url
  comic.websiteUrl = tom_date_to_website_url(comic.identifier);
  
  // more debug info
  tom_debug_message("init: comic.identifierSpecified="+comic.identifierSpecified); 
  tom_debug_message("init: comic.identifier="+comic.identifier);
  tom_debug_message("init: comic.firstIdentifier="+comic.firstIdentifier);
  tom_debug_message("init: comic.nextIdentifier="+comic.nextIdentifier);
  tom_debug_message("init: comic.previousIdentifier="+comic.previousIdentifier);
  tom_debug_message("init: comic.websiteUrl="+comic.websiteUrl);

  // request the html page
  comic.requestPage(comic.websiteUrl, comic.Page);
}

/**
 * Handle the received data.
 */
function pageRetrieved(id, data) {
  tom_debug_message("function pageRetrieved(id="+tom_page_retrieved_id_to_string(id)+",data)");

  if (id == comic.Page) { 
    // search for the element: <img src="/dx/.1/gif.t,tom.d,1253311200" alt="TOM" width="560" />
    // and extract the value of the src-attribute
    var re = new RegExp("<img[\\s]+src=\"(/dx/.1/gif.t,tom.d,[\\d]+)\"");
    var match = re.exec(data);
    // handle error
    if (match == null) {
      tom_debug_message("error:could not extract imageURL from html source\nhtml:\n"+data);
      comic.error();
      return;
    }
    // the tom-comic image URL linked by the current website
    var imageURL = "http://www.taz.de/" + match[1];
    tom_debug_message("imageURL:"+imageURL);
    // request the image
    comic.requestPage(imageURL, comic.Image, tom_get_client_info(comic.websiteUrl));
  } else if (id == comic.Image) {
    if (TOM_DEBUG_ENABLED && data.rawData == null) {
      // no comics on sundays and national holidays 
      tom_debug_message("image: null");
     // comic.error();
    }
  }
}
